function vlsm2(design, imagedir, outdir, varargin)

% VLSM2  Voxel-based Lesion-Symptom Mapping.
%
% Version 2.55
% 
% Copyright 2002 - 2014, Stephen M. Wilson, University of Arizona
% 
% If you use VLSM, please cite:
%
% Bates E, Wilson SM, Saygin AP, Dick F, Sereno MI, Knight RT, Dronkers NF.
% Voxel-based lesion-symptom mapping. Nat Neurosci 2003; 6: 448-50.
%
% Software available at: http://neuroling.arizona.edu/resources.html
%
% VLSM2(DESIGN, IMAGEDIR, OUTDIR, param, value, ...) performs a VLSM
% analysis using the design matrix in DESIGN and the images in IMAGEDIR,
% saving the outputs in the directory OUTDIR.
% 
% DESIGN is the name of a tab-delimited text file which specifies the
% design matrix for the study. Rows correspond to subjects, and columns
% correspond to potential explanatory variables. There should be a header
% row. One column must have the name "Filename", and contain image
% filenames. If filename extensions are omitted, '.nii' and '.img'
% extensions will be tried. Alternatively, a "template" for filenames can
% be provided with the 'fnametemplate' option (see below).
% 
% IMAGEDIR is the directory containing images. If empty, then image
% filenames will be interpreted relative to the working directory when the
% program is run.
% 
% OUTDIR is the output directory. If it does not exist, it is created. An
% HTML file is created in OUTDIR, summarizing the results. Also in OUTDIR,
% a file named roi.txt is created that consists of the included rows of the
% design file, with mean lesion values in each cluster as well as at each
% peak, and the values actually used in the design matrix. This file can be
% used for post-hoc analysis.
% 
% Possible param/value options are:
% 
% 'vars' a cell array of variables to use in the analysis. These should
% match column names in the design file. The first variable provided is the
% effect of interest; all others are interpreted as covariates. If 'vars'
% is not specified, then the default is to use all variables from the
% design file, in order.
%   Empty values are ignored. Values of inf or -inf are allowed but you
% also need to use 'clipoutliers', to clip these values to the min or max
% respectively.
%   The special variable 'LesionSize' (case-insensitive) does not need to
% be included in the design file, but rather will be automatically
% calculated from the images.
%
% 'include' the name of a column that specifies which subjects are included.
% In that column, 1 = include and 0 = exclude. Default is to include all
% subjects.
%
% 'clipoutliers' is a vector of which variables in 'vars' should be clipped
% at a certain distance from the mean. If numbers are provided, they are in
% units of standard deviations, whereas 0 means don't clip.
% 
% 'mode' can be 'vlsm' (default) or 'vbm'. In 'vlsm' mode, lesions are
% denoted by 1s and intact tissue by 0s (possibly with smoothing to create
% intermediate values). In 'vbm' mode, voxels reflect gray matter
% proportions.
% 
% 'alpha' is a vector of voxelwise alpha values to consider. The default is
% [0.01 0.005 0.001]. These are used for (i) power analysis; (ii)
% thresholding. Real and scrambled data are thresholded at T statistics
% corresponding to these P values, and resulting cluster sizes are
% compared.
% 
% 'mask' is an explicit masking image.
%
% 'maskthresh' in 'vlsm' mode is the minimum number of subjects lesioned at
% a voxel for that voxel to be included in the analysis, and in 'vbm' mode
% is the minimum mean GM proportion for a voxel to be included. The default
% is 10 for 'vlsm' and 0.05 for 'vbm'.
% 
% 'maskpower' is the minimum power for a voxel to be included in the
% analysis. This is based on the first P value in 'alpha', unless
% 'maskpower' is a vector, in which case the second element of 'maskpower'
% tells which element of 'alpha' to use, e.g. 'maskpower' = [0.4 2] would
% restrict the analysis to voxels with a power of at least 0.4 to detect an
% effect at p < 0.005 (since 0.005 is the 2nd element of alpha). The
% default is [], which implies no masking according to power.
% 
% 'nperms' is the number of permutations to run to determine distribution
% of maximum Ts and cluster sizes. The default is 0, i.e. no permutation
% analysis. If you want to use the null distribution from a previous
% analysis, you can pass the filename here instead (e.g. 'nulldist.txt').
% 
% 'permvars' is a binary vector describing which of the variables in 'vars'
% to permute. Normally, behavioral measures would be permuted, whereas
% variables related to the brains (e.g. age, sex) would not. The default is
% [1 0 0 ... 0], i.e. permute only the first variable.
% 
% 'fnametemplate' is a template for picking out filenames, where the
% filename from the design file is slotted in at the point marked '%s'. For
% example, if your files were smoothed NIFTI images with 's' prefixes, but
% your design file contained only the stems of the names, you could set
% 'fnametemplate' = 's%s.nii'.
% 
% 'power_d' is the effect size for the power analysis. The default is 0.8.
% 
% 'flip', if set to 1/true, flips images L-R prior to analysis. VLSM now
% expects images to be in RPI format (as for SPM2, SPM5 and FSL). If your
% images are in LPI format, you can use this option.
%
% 'highscoresarebad', if set to 1/true, implies that higher values of the
% behavioral measure imply worse performance. Note that for covariates, the
% direction never matters.
%
% 'underlay' is an image to use as an underlay. The default is the MNI high
% resolution single subject image.


%% startup

fprintf('_______________________________________________________________\n\n');
fprintf('VLSM2  Voxel-based Lesion-Symptom Mapping\n\n');
fprintf('Version 2.55\n\n');
fprintf('Copyright 2002 - 2014, Stephen M. Wilson, University of Arizona\n');
fprintf('_______________________________________________________________\n\n');


%% argument processing

if nargin < 3
  error('Not enough input arguments.');
end

v = vlsm_processargs(varargin, ...
  'vars', 'c', [], ...
  'include', 's', '', ...
  'clipoutliers', 'n', [], ...
  'mode', 's', 'vlsm', ...
  'alpha', 'n', [0.01 0.005 0.001], ...
  'mask', 's', '', ...
  'maskthresh', 'n', 10, ...
  'maskpower', 'n', [], ...
  'nperms', '', 0, ...
  'permvars', 'n', [], ...
  'fnametemplate', 's', [], ...
  'power_d', 'n', 0.8, ...
  'flip', 'b', 0, ...
  'highscoresarebad', 'b', 0, ...
  'underlay', 's', '');

if strcmp(v.mode, 'vbm') && v.maskthresh == 10
  v.maskthresh = 0.05;
end

mfname = mfilename;
mpath = which(mfname);
mpath = fileparts(mpath);

if strcmp(v.underlay, '')
  v.underlay = fullfile(mpath, '_colin.img');
else
  v.underlay = GetFullPath(v.underlay); % because it will be used from a different working directory
end


%% initialization

starttime = datestr(now);

wd = pwd;

if exist(outdir, 'dir')
  answer = input('Overwrite existing directory? ', 's');
  if ~isempty(answer) && lower(answer(1)) == 'y'
    success = rmdir(outdir, 's');
    if ~success
      warning('Existing directory not successfully deleted.'); %#ok<WNTAG>
    end
  else
    return;
  end
end

mkdir(outdir);
cd(outdir);

logfname = ['log_' starttime '.txt'];
logfname(logfname == '-' | logfname ==' ' | logfname == ':') = '_';
diary(logfname);

if exist('vlsm.html', 'file')
  delete('vlsm.html');
end
fid = fopen('vlsm.html', 'w');
c1 = onCleanup(@()fclose(fid));

if exist('roi.txt', 'file')
  delete('roi.txt');
end
roifid = fopen('roi.txt', 'w');
c2 = onCleanup(@()fclose(roifid));

if exist('vlsm_files', 'dir')
  rmdir('vlsm_files', 's');
end
mkdir('vlsm_files');

fprintf(fid, '<HTML>\n<HEAD>\n<TITLE>VLSM %s</TITLE>\n</HEAD>\n\n', outdir);
fprintf(fid, '<BODY>\n\n');

fprintf(fid, '<FONT FACE="Georgia, Times" SIZE = +3>\n');
fprintf(fid, '<P><B><I>vlsm</I></B></P>\n');
fprintf(fid, '</FONT>\n');
fprintf(fid, '<FONT FACE="Verdana, Arial, Helvetica">\n');
fprintf(fid, '<P>Voxel-based Lesion-Symptom Mapping, version 2.55</P>\n');

imwrite(permute(repmat(jet(100), [1 1 15]), [3 1 2]),'vlsm_files/colorbar.jpg')


%% read design

% read design file
cd(wd);
fprintf('Reading design file %s.\n', design);
designrows = textread(design, '%s', 'delimiter', '\n', 'bufsize', 32767);
nrows = length(designrows);
n = nrows - 1;
colnames = strread(designrows{1}, '%s', 'delimiter', '\t');
ncols = length(colnames);
designcells = cell(n, ncols);
for r = 1:n
  nextcells = strread(designrows{r + 1}, '%s', 'delimiter', '\t')';
  designcells(r, 1:length(nextcells)) = nextcells;
end
fprintf('Found %d subjects and %d columns in design file.\n', n, ncols);

% find filename column
fnamecol = 0;
for c = 1:ncols
  if strcmpi(colnames{c}, 'Filename')
    fnamecol = c;
    break;
  end
end
if fnamecol == 0
  error('Design must contain a Filename column.');
end

% if vars not provided, use all columns as vars
if isempty(v.vars)
  v.vars = colnames(range_except(1:ncols, fnamecol));
  fprintf('No variables provided, so using all columns as variables.\n');
end

% which subjects to explicitly include or not
include = ones(n, 1);
if ~strcmp(v.include, '')
  includecol = 0;
  for c = 1:ncols
    if strcmpi(colnames{c}, v.include)
      includecol = c;
      break;
    end
  end
  if includecol == 0
    error('Could not find column %s indicating subject to be included.', v.include);
  end
  for r = 1:n
    val = str2double(designcells{r, includecol});
    if val
      include(r) = 1;
    else
      include(r) = 0;
    end
  end
end

% construct x from vars
nvars = length(v.vars);
lesion_size_cov = 0;
x = nan(n, nvars + 1);
for vr = 1:nvars
  varcol = 0;
  for c = 1:ncols
    if strcmpi(colnames{c}, v.vars{vr})
      varcol = c;
      break;
    end
  end
  if varcol == 0
    if strcmpi(v.vars{vr}, 'lesionsize')
      lesion_size_cov = vr;
      x(:, vr) = 0;
      fprintf('Will automatically calculate and covary out lesion size.\n');
    else
      error('Variable %s not found in design.', v.vars{vr});
    end
  else
    for r = 1:n
      val = str2double(designcells{r, varcol});
      if ~isempty(val)
        x(r, vr) = val;
      end
      if include(r) && (isnan(val) || isempty(val))
        fprintf('Warning: variable %s missing for subject %d with image filename %s.\n', ...
          v.vars{vr}, r, designcells{r, fnamecol});
      end
    end
  end
end
x(:, nvars + 1) = 1;

include = include & all(~isnan(x), 2);
fprintf('Of these, %d are included and have all required variables present.\n', sum(include));


%% read images

% which images exist?
if ~isempty(imagedir)
  cd(imagedir);
end
fnames = cell(n, 1);
origfnames = cell(n, 1);
count = 0;
for i = 1:n
  if include(i)
    fname = designcells{i, fnamecol};
    while fname(end) == ' '
      fname = fname(1:end - 1);
    end
    if ~isempty(v.fnametemplate)
      fname = sprintf(v.fnametemplate, fname);
    end    
    if exist(fname, 'file')
      fname = fname; %#ok<ASGSL>
    elseif exist([fname '.nii'], 'file')
      fname = [fname '.nii']; %#ok<AGROW>
    elseif exist([fname, '.img'], 'file');
      fname = [fname '.img']; %#ok<AGROW>
    else
      fname = [];
    end
    if ~isempty(fname)
      count = count + 1;
      fnames{count} = fname;
      origfnames{count} = designcells{i, fnamecol};
    else
      include(i) = 0;
      fprintf('Warning: could not find image %s.\n', designcells{i, fnamecol});
    end
  end
end

x = x(include, :);
designcells = designcells(include, :);
n = size(x, 1);
fprintf('Of these, %d also have images.\n', n);

% read first image to get header info
fprintf('Reading first image...\n');
[hdr, img] = read_nifti(fnames{1});
dim = hdr.dim(2:4);
nvox = prod(dim);
fprintf('Dimensions of first image: %d x %d x %d, i.e. %d voxels.\n', dim(1), dim(2), dim(3), nvox);
fprintf('Image storage will require %d MB.\n', round(nvox * n * 2 / (2 ^ 20)));

% make a generic header to be used for writing
genhdr = make_nifti_hdr('float32', fnames{1});
genhdr.magic = 'n+1 ';
genhdr.magic(4) = 0;
genhdr.vox_offset = 352;

% allocate memory and read images
fprintf('Allocating memory for images...\n');
y = zeros(n, nvox, 'int16');
fprintf('Reading images...\n');
for i = 1:n
  fprintf('.');
  if i >= 2
    [hdr, img] = read_nifti(fnames{i}); %#ok<ASGLU>
    if any(isnan(img(:)))
      fprintf('Warning: image %s contains NaNs, which are being converted to zeros.\n', fnames{i});
      img(isnan(img)) = 0;
    end
  end
  if v.flip
    img = flipdim(img, 1);
  end
  if lesion_size_cov
    lesion_size = sum(img(:));
    x(i, lesion_size_cov) = lesion_size;
  end
  y(i, :) = round(img(:) * 10000);
end
fprintf('\n');


%% clip outliers

if ~isempty(v.clipoutliers)
  for i = 1:nvars
    if length(v.clipoutliers) >= i && v.clipoutliers(i) > 0
      xmean = mean(x(x(:, i) ~= inf & x(:, i) ~= -inf, i));
      xsd = std(x(x(:, i) ~= inf & x(:, i) ~= -inf, i));
      clipmax = xmean + xsd * v.clipoutliers(i);
      clipmin = xmean - xsd * v.clipoutliers(i);
      x(x(:, i) > clipmax) = clipmax;
      x(x(:, i) < clipmin) = clipmin;
    end
  end
end


%% write design summary

cd(wd);
cd(outdir);

fprintf(fid, '<P>&nbsp;</P><HR>\n<P><B>Design</B></P>\n');

fprintf(fid, '<P>%d subjects included: ', n);
for i = 1:n
  fprintf(fid, '%s ', origfnames{i});
end
fprintf(fid, '</P>\n');
fprintf(fid, '<P>%d variables: ', nvars);
for i = 1:nvars
  fprintf(fid, '%s ', v.vars{i});
  if length(v.clipoutliers) >= i && v.clipoutliers(i) > 0
    fprintf(fid, '(clipped at %.1f s.d.s from mean: %.2f to %.2f) ', v.clipoutliers(i), clipmin, clipmax);
  end
end
fprintf(fid, '</P>\n');

figure('Units', 'inches', 'Position', [2 2 8 2]);
imx = x - repmat(mean(x, 1), n, 1);
imx = imx ./ repmat(max(imx), n, 1);
imx = 50 + imx * 50;
image(imx(:, 1:nvars)');
axis off;
set(gca, 'Position', [0 0 1 1]);
colormap(jet(100));
set(gcf, 'PaperPositionMode', 'auto');
print('-djpeg', '-r0', 'vlsm_files/desmat.jpg');
close(gcf);

fprintf(fid, '<IMG SRC="vlsm_files/desmat.jpg"></P>\n');
fprintf(fid, '<P></P>\n');


%% overlay

if strcmp(v.mode, 'vlsm')
  overlay = sum(y > 10000 / 2, 1);
  maxlesioned = max(overlay);
  fprintf('Maximum patients lesioned in a voxel: %d\n', maxlesioned);
else
  overlay = mean(y, 1) / 10000;
end
write_nifti_masked(genhdr, overlay, 'overlay.nii', true(1, nvox));

% write HTML
fprintf(fid, '<P>&nbsp;</P><HR>\n<P><B>Overlay</B></P>\n');
slices = getslices('aaasssc', [-15 15 45 -50 0 50 0], [1 7]);
images(1).fname = v.underlay;
images(2).fname = 'overlay.nii';
images(2).cmap = 'jet';
if strcmp(v.mode, 'vlsm')
  images(2).scale = [1, max(overlay)];
else
  images(2).scale = [0.1, 1];
end
options.printwidth = 12;
options.crop = [5 5 5 5 0 10];
options.closewindow = true;
options.gridlines = false;
multiview('vlsm_files/overlay', slices, images, options);
fprintf(fid, '<IMG SRC="vlsm_files/overlay.jpg"></P>\n');
if strcmp(v.mode, 'vlsm')
  fprintf(fid, '<P>Number of patients with lesions including each voxel&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%g <IMG SRC="vlsm_files/colorbar.jpg"> %g</P>\n', 1, maxlesioned);
end


%% power analysis

nalphas = length(v.alpha);
if strcmp(v.mode, 'vlsm')
  fprintf('Performing power analysis...\n');

  fprintf(fid, '<P>&nbsp;</P><HR>\n<P><B>Power analysis</B></P>\n');

  powermap = cell(nalphas, 1);
  for a = 1:nalphas
    za = qnorm(1 - v.alpha(a));

    power = zeros(maxlesioned, 1);

    for na = 1:maxlesioned
      nb = n - na;
      n_ = (2 * na * nb) / (na + nb); % harmonic mean (Cohen 1977, p. 42)
      power(na) = pnorm((v.power_d * (n_ - 1) * sqrt(2 * n_)) / (2 * (n_ - 1) + 1.21 * (za - 1.06)) - za);
        % Dixon-Massey approximation (Cohen 1977, p. 456)
        % note there is a typo in the book: the last 'b' should be an 'a'
    end
    fprintf('For alpha = %g, maximum power = %.2f\n', v.alpha(a), power(end));

    power = [0; power]; %#ok<AGROW>
    powermap{a} = power(overlay + 1);
    write_nifti_masked(genhdr, powermap{a}, sprintf('power_%g.nii', v.alpha(a)), true(1, nvox));

    images(2).fname = sprintf('power_%g.nii', v.alpha(a));
    images(2).scale = [0.4 0.8];
    multiview(sprintf('vlsm_files/power_%g', v.alpha(a)), slices, images, options);
    fprintf(fid, '<IMG SRC="vlsm_files/power_%g.jpg"></P>\n', v.alpha(a));  
    fprintf(fid, '<P>Power map for alpha = %g&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%g <IMG SRC="vlsm_files/colorbar.jpg"> %g</P>\n', v.alpha(a), 0.4, 0.8);
  end
  fprintf(fid, '<P></P>\n');
end


%% masking

mask = overlay > 0;
fprintf('Number of voxels which are not all zero: %d\n', sum(mask));

% explicit mask
if ~isempty(v.mask)
  [maskhdr, maskimg] = read_nifti(v.mask); %#ok<ASGLU>
  if any(isnan(maskimg(:)))
    fprintf('Warning: mask image %s contains NaNs, which are being converted to zeros.\n', v.mask);
    maskimg(isnan(maskimg)) = 0;
  end
  mask = mask & maskimg(:)';
end

% explicit minimum (number of subjects for VLSM, minimum GM proportion for VBM)
if ~isempty(v.maskthresh)
  mask = mask & (overlay >= v.maskthresh);
  if strcmp(v.mode, 'vlsm')
    mask = mask & (overlay <= (n - v.maskthresh));
  end
end

% mask based on power
if ~isempty(v.maskpower)
  if length(v.maskpower) == 2
    powi = v.maskpower(2);
  else
    powi = 1;
  end
  mask = mask & powermap{powi}' >= v.maskpower(1);
end

nvox = sum(mask);
fprintf('Number of voxels in all masks: %d\n', nvox);
if ~nvox
  error('No voxels survived masking.');
end
y = y(:, mask);

if v.highscoresarebad
  y = -y;
end

if strcmp(v.mode, 'vlsm')
  y = -y; % so that negative values correspond to tissue loss
end


%% fit model

fprintf('Fitting model to data, voxel by voxel...\n');
[timg, efimg, sdimg] = fit_model(x, y);
max_t = max(timg);
fprintf('Maximum T in real data = %.3f\n', max_t);

write_nifti_masked(genhdr, timg, 't.nii', mask);
write_nifti_masked(genhdr, efimg, 'ef.nii', mask);
write_nifti_masked(genhdr, sdimg, 'sd.nii', mask);

fprintf(fid, '<P>&nbsp;</P><HR>\n<P><B>Statistics</B></P>\n');
images(2).fname = 't.nii';
images(2).scale = [0.0001 max_t];
multiview('vlsm_files/t', slices, images, options);
fprintf(fid, '<IMG SRC="vlsm_files/t.jpg"></P>\n');  
fprintf(fid, '<P>Raw T map&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%g <IMG SRC="vlsm_files/colorbar.jpg"> %.2f</P>\n', 0, max_t);

% write voxelwise thresholded T images
df = n - nvars - 1;
crit_t = qt(1 - v.alpha, df);

for a = 1:nalphas
  tthrimg = timg .* (timg >= crit_t(a));
  write_nifti_masked(genhdr, tthrimg, sprintf('t_%g.nii', v.alpha(a)), mask);  

  images(2).fname = sprintf('t_%g.nii', v.alpha(a));
  images(2).scale = [crit_t(a) max(max_t, crit_t(a))];
  multiview(sprintf('vlsm_files/t_%g', v.alpha(a)), slices, images, options);
  fprintf(fid, '<IMG SRC="vlsm_files/t_%g.jpg"></P>\n', v.alpha(a));
  fprintf(fid, '<P>T map with voxelwise threshold of p < %g&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%.2f <IMG SRC="vlsm_files/colorbar.jpg"> %.2f</P>\n', v.alpha(a), images(2).scale(1), images(2).scale(2));
  
  [real_cimg{a}, real_ctab{a}, peaks] = cluster('t.nii', crit_t(a)); %#ok<AGROW>
  real_cimg{a} = real_cimg{a}(:)'; %#ok<AGROW>
  real_cimg{a} = real_cimg{a}(mask); %#ok<AGROW>

  if ~isempty(real_ctab{a})
    nclusters = size(real_ctab{a}, 1);
    fprintf('For alpha = %g, there are %d clusters with volumes: ', v.alpha(a), nclusters);
    fprintf('%d ', real_ctab{a}(:, 2));
    fprintf('\n');
    
    % write HTML
    fprintf(fid, '<TABLE BORDER="1"><TR><TH>Cluster</TH><TH>Volume</TH><TH>px</TH><TH>py</TH><TH>pz</TH><TH>cx</TH><TH>cy</TH><TH>cz</TH><TH>Max T</TH></TR>\n');
    for i = 1:size(real_ctab{a}, 1)
      fprintf(fid, '<TR><TD>%d</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.2f</TD></TR>\n', ...
        i, real_ctab{a}(i, 2), real_ctab{a}(i, 11), real_ctab{a}(i, 12), real_ctab{a}(i, 13), ...
        real_ctab{a}(i, 14), real_ctab{a}(i, 15), real_ctab{a}(i, 16), real_ctab{a}(i, 4));
    end
    fprintf(fid, '</TABLE><BR>\n');
    
    % write roi.txt file for ROI analysis
    fprintf(roifid, 'For alpha = %g, cluster sizes are: ', v.alpha(a));
    fprintf(roifid, '%d ', real_ctab{a}(:, 2));
    fprintf(roifid, '\n');

    peakimg = zeros(genhdr.dim(2:4));    
    for c = 1:nclusters
      peakimg(peaks(c, 1), peaks(c, 2), peaks(c, 3)) = c;
    end
    peakimg = peakimg(:)';
    peakimg = peakimg(mask);

    fprintf(roifid, '%s\t', colnames{:});
    for c = 1:nclusters
      fprintf(roifid, 'clust%d_mean\tclust%d_peak\t', c, c);
    end
    for vr = 1:nvars
      fprintf(roifid, 'x_%s\t', v.vars{vr});
    end
    fprintf(roifid, '\n');
    
    for i = 1:n
      fprintf(roifid, '%s\t', designcells{i, :}); % design
      for c = 1:nclusters
        meanval = mean(double(y(i, real_cimg{a} == c))) / 10000;
        peakval = double(y(i, peakimg == c)) / 10000;
        fprintf(roifid, '%.3f\t%.3f\t', meanval, peakval);
      end
      fprintf(roifid, '%.0f\t', x(i, 1:end - 1)); % design matrix
      fprintf(roifid, '\n');
    end
    fprintf(roifid, '\n');
  end
end
fprintf(fid, '<P></P>\n');


%% permutations

final_alpha = 0.05;
if isnumeric(v.nperms) && v.nperms == 0
  fprintf('No permutation analysis.\n');
else
  % if not specified, permute only the first variable
  if isempty(v.permvars)
    v.permvars = zeros(1, nvars);
    v.permvars(1) = 1;
  end
  v.permvars = logical(v.permvars);

  if isnumeric(v.nperms)
    fprintf('Testing signifiance based on %d permutations...\n', v.nperms);

    % do the permutations
    nulldist = zeros(v.nperms, nalphas + 1);
    if false
      v_permvars = v.permvars;
      nd = cell(v.nperms, 1);
      for p = 1:v.nperms
        nd{p} = zeros(1, nalphas + 1);
      end
      for p = 1:v.nperms
        fprintf('%d ', p);
        x_ = x;
        x_(:, v_permvars) = x_(randperm(n), v_permvars);
        t_img = fit_model(x_, y);

        max_t_ = max(t_img);    
        nd{p}(1) = max_t_;
        fprintf('%.3f ', max_t_);

        write_nifti_masked(genhdr, t_img, sprintf('t%05d_.nii', p), mask);
        % write_nifti_masked(genhdr, t_img, sprintf('t_%d.nii', p), mask);
        for a = 1:nalphas
          [cimg, ctab] = cluster(sprintf('t%05d_.nii', p), crit_t(a)); %#ok<ASGLU>
          if isempty(ctab)
            max_clust = 0;
          else
            max_clust = ctab(1, 2);
          end
          nd{p}(a + 1) = max_clust;
          fprintf('%d ', max_clust);
        end
        fprintf('\n');
        delete(sprintf('t%05d_.nii', p));
      end
      for p = 1:v.nperms
        nulldist(p, :) = nd{p};
      end      
    else
      for p = 1:v.nperms
        fprintf('%d ', p);
        x_ = x;
        x_(:, v.permvars) = x_(randperm(n), v.permvars);
        t_img = fit_model(x_, y);

        max_t_ = max(t_img);    
        nulldist(p, 1) = max_t_;
        fprintf('%.3f ', max_t_);

        write_nifti_masked(genhdr, t_img, 't_.nii', mask);
        % write_nifti_masked(genhdr, t_img, sprintf('t_%d.nii', p), mask);
        for a = 1:nalphas
          [cimg, ctab] = cluster('t_.nii', crit_t(a)); %#ok<ASGLU>
          if isempty(ctab)
            max_clust = 0;
          else
            max_clust = ctab(1, 2);
          end
          nulldist(p, a + 1) = max_clust;
          fprintf('%d ', max_clust);
        end
        fprintf('\n');
      end
      delete('t_.nii');
    end

    % find cutoffs
    for i = 1:nalphas + 1
      nulldist(:, i) = sort(nulldist(:, i), 'descend');
    end
    
    % save null distribution to file
    ndfid = fopen('nulldist.txt', 'w');
    for i = 1:size(nulldist, 1)
      fprintf(ndfid, '%.4f', nulldist(i, 1));
      for j = 1:nalphas
        fprintf(ndfid, '\t%d', nulldist(i, j + 1));
      end
      fprintf(ndfid, '\n');
    end
    fclose(ndfid);

  else
    fprintf('Testing signifiance based on null matrix provided...\n');
    nulldist = dlmread(v.nperms);
    v.nperms = size(nulldist, 1);
  end
  
  cutoff = floor(final_alpha * v.nperms);
  if cutoff == 0
    fprintf('WARNING: Insufficient permutations to actually determine significance.\n');
    cutoff = 1;
  end
  perm_crit_t = nulldist(cutoff, 1);
  fprintf('Max T threshold = %.2f\n', perm_crit_t);

  % write max-T-thresholded images and cluster table
  tthrimg = timg .* (timg >= perm_crit_t);
  write_nifti_masked(genhdr, tthrimg, 't_thresh.nii', mask);

  fprintf(fid, '<P>&nbsp;</P><HR>\n<P><B>Permutation-based thresholding</B></P>\n');
  images(2).fname = 't_thresh.nii';
  images(2).scale = [perm_crit_t max(max_t, perm_crit_t)];
  multiview('vlsm_files/t_thresh', slices, images, options);
  fprintf(fid, '<IMG SRC="vlsm_files/t_thresh.jpg"></P>\n');
  fprintf(fid, '<P>T map with voxelwise threshold of T > %.2f, which is the critical T based on the permutation method&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%.2f <IMG SRC="vlsm_files/colorbar.jpg"> %.2f</P>\n', perm_crit_t, images(2).scale(1), images(2).scale(2));  
  
  [real_cimg_t, real_ctab_t] = cluster('t.nii', perm_crit_t); %#ok<ASGLU>
  if ~isempty(real_ctab_t)
    fprintf(fid, '<TABLE BORDER="1"><TR><TH>Cluster</TH><TH>Volume</TH><TH>px</TH><TH>py</TH><TH>pz</TH><TH>cx</TH><TH>cy</TH><TH>cz</TH><TH>Max T</TH></TR>\n');
    for i = 1:size(real_ctab_t, 1)
      fprintf(fid, '<TR><TD>%d</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.2f</TD></TR>\n', ...
        i, real_ctab_t(i, 2), real_ctab_t(i, 11), real_ctab_t(i, 12), real_ctab_t(i, 13), ...
        real_ctab_t(i, 14), real_ctab_t(i, 15), real_ctab_t(i, 16), real_ctab_t(i, 4));
    end
    fprintf(fid, '</TABLE><BR>\n');
  end
  
  % write cluster-size-thresholded images and tables
  for a = 1:nalphas
    last_clust_idx = 0;
    if ~isempty(real_ctab{a})
      for i = 1:size(real_ctab{a}, 1)
        pval = min(find(nulldist(:, a + 1) < real_ctab{a}(i, 2))) / v.nperms; %#ok<MXFND>
        if isempty(pval)
          pval = 1;
        end
        real_ctab{a}(i, 3) = pval; %#ok<AGROW>
        if pval <= final_alpha
          last_clust_idx = i;
        end
      end
    end
    
    tthrimg = timg .* (real_cimg{a} >= 1 & real_cimg{a} <= last_clust_idx);
    write_nifti_masked(genhdr, tthrimg, sprintf('t_%g_thresh.nii', v.alpha(a)), mask);
    
    images(2).fname = sprintf('t_%g_thresh.nii', v.alpha(a));
    images(2).scale = [crit_t(a) max(max_t, crit_t(a))];
    multiview(sprintf('vlsm_files/t_%g_thresh', v.alpha(a)), slices, images, options);
    fprintf(fid, '<IMG SRC="vlsm_files/t_%g_thresh.jpg"></P>\n', v.alpha(a));
    fprintf(fid, '<P>T map with voxelwise threshold of p < %g, thresholded based on cluster size and the permutation method&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%.2f <IMG SRC="vlsm_files/colorbar.jpg"> %.2f</P>\n', v.alpha(a), images(2).scale(1), images(2).scale(2));

    if ~isempty(real_ctab{a})
      fprintf(fid, '<TABLE BORDER="1"><TR><TH>Cluster</TH><TH>Volume</TH><TH>px</TH><TH>py</TH><TH>pz</TH><TH>cx</TH><TH>cy</TH><TH>cz</TH><TH>Max T</TH><TH>Corrected P</TH></TR>\n');
      for i = 1:size(real_ctab{a}, 1)
        fprintf(fid, '<TR><TD>%d</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.0f</TD><TD>%.2f</TD><TD>%.4f</TD></TR>\n', ...
          i, real_ctab{a}(i, 2), real_ctab{a}(i, 11), real_ctab{a}(i, 12), real_ctab{a}(i, 13), ...
          real_ctab{a}(i, 14), real_ctab{a}(i, 15), real_ctab{a}(i, 16), real_ctab{a}(i, 4), real_ctab{a}(i, 3));
      end
      fprintf(fid, '</TABLE><BR>\n');
    end
  end
end


%% cleanup

fprintf(fid, '<P>&nbsp;</P><HR>\n');
fprintf(fid, '</BODY>\n</HTML>\n');
% fclose(fid);
% fclose(roifid);

cd(wd);

diary off;


%% fit model

function [timg, efimg, sdimg] = fit_model(x, y_in)

% parallelized by Andrew T. DeMarco, 2011

nvox = size(y_in, 2);

timg = zeros(1, nvox);
efimg = zeros(1, nvox);
sdimg = zeros(1, nvox);

[Q, R] = qr(x, 0);

% if exist('parfor', 'builtin') % parallel for loop
%   parfor i = 1:nvox
%     y = double(y_in(:, i)) / 10000;
%     beta = R \ (Q' * y);
%     yhat = x * beta;
%     residuals = y - yhat;
%     n = length(y);
%     % ymean = sum(y) / n;
%     pp = min(size(R));
%     sse = sum(residuals .* residuals);
%     dfe = n - pp;
%     mse = sse / dfe;
%     % ssr = sum((yhat - ymean) .* (yhat - ymean));
%     % dfr = pp - 1;
%     % msr = ssr / dfr;
%     % overallf = msr / mse;  
%     ri = R \ eye(pp);
%     xtxi = ri * ri';
%     betacov = xtxi * mse;
%     betavar = diag(betacov);
%     betasd = sqrt(betavar);
%     t = beta ./ betasd;
%     % f = t .^ 2;
%     timg(i) = t(1);
%     sdimg(i) = mse; % betasd(1);
%   end
% else % sequential (non-parallel) for loop
  for i = 1:nvox
    if mod(i, 5000) == 1, fprintf('.'); end
    y = double(y_in(:, i)) / 10000;
    beta = R \ (Q' * y);
    yhat = x * beta;
    residuals = y - yhat;
    n = length(y);
    % ymean = sum(y) / n;
    pp = min(size(R));
    sse = sum(residuals .* residuals);
    dfe = n - pp;
    mse = sse / dfe;
    % ssr = sum((yhat - ymean) .* (yhat - ymean));
    % dfr = pp - 1;
    % msr = ssr / dfr;
    % overallf = msr / mse;  
    ri = R \ eye(pp);
    xtxi = ri * ri';
    betacov = xtxi * mse;
    betavar = diag(betacov);
    betasd = sqrt(betavar);
    t = beta ./ betasd;
    % f = t .^ 2;
    timg(i) = t(1);
    efimg(i) = beta(1);
    sdimg(i) = mse; % betasd(1);
  end
  fprintf('\n');
% end
