function vlsm_preproc(mri_fname, roi_fname)

% VLSM_PREPROC(MRI_FNAME, ROI_FNAME)
%
% Warps MRI_FNAME to MNI space using Unified Segmentation. ROI_FNAME can be
% a single mask image or a cell array of related mask images. All mask
% images are smoothed. The first mask image is dilated and used for cost
% function masking in the warp. All mask images, and the anatomical image,
% are written to MNI space.
%
% All images must be in .nii format.
%
% Copyright 2012 Stephen M. Wilson, University of Arizona


if ~strcmp(spm('Ver'), 'SPM5')
  error('This script requires SPM5.');
end

spm_defaults;

if ~iscell(roi_fname)
  in_roi_fname = roi_fname;
  clear roi_fname;
  roi_fname{1} = in_roi_fname;
end
nrois = numel(roi_fname);

% check anatomical and copy
[path, name, ext] = fileparts(mri_fname);
if ~strcmp(ext, '.nii')
  error('Anatomical image must be a .nii file.');
end
copyfile(fullfile(path, [name ext]), fullfile(path, ['r' name ext]));

% check file type of ROIs and copy
for r = 1:nrois
  % check file type
  [path, name, ext] = fileparts(roi_fname{r});
  if ~strcmp(ext, '.nii')
    error('ROIs must be .nii files.');
  end
  copyfile(fullfile(path, [name ext]), fullfile(path, ['r' name ext]));
end

% coreg anatomical to colin
clear jobs;
jobs{1}.spatial{1}.coreg{1}.estimate.ref{1} = fullfile(fileparts(which('spm')), 'canonical', 'single_subj_T1.nii');
[path, name, ext] = fileparts(mri_fname);
jobs{1}.spatial{1}.coreg{1}.estimate.source{1} = fullfile(path, ['r' name ext]);
for r = 1:nrois
  [path, name, ext] = fileparts(roi_fname{r});
  jobs{1}.spatial{1}.coreg{1}.estimate.other{r} = fullfile(path, ['r' name ext]); %#ok<AGROW>
  jobs{1}.spatial{1}.coreg{1}.estimate.eoptions.sep = [16 8 4 2];
end
spm_jobman('run_nogui', jobs);

% make sure each ROI = 255
for r = 1:nrois
  [path, name, ext] = fileparts(roi_fname{r});
  [hdr, img] = read_nifti(fullfile(path, ['r' name ext]));
  img = img ./ max(img(:)) * 255;
  write_nifti(hdr, img, fullfile(path, ['br' name ext]));
end

% smooth each ROI
clear jobs;
for r = 1:nrois
  [path, name, ext] = fileparts(roi_fname{r});
  jobs{1}.spatial{1}.smooth.data{r} = fullfile(path, ['br' name ext]); %#ok<AGROW>
end
jobs{1}.spatial{1}.smooth.fwhm = [4 4 4];
spm_jobman('run_nogui', jobs);

% reslice each ROI and dilate the first to create the cost function mask
clear jobs;
[path, name, ext] = fileparts(mri_fname);
jobs{1}.spatial{1}.coreg{1}.write.ref{1} = fullfile(path, ['r' name ext]);
for r = 1:nrois
  [path, name, ext] = fileparts(roi_fname{r});
  jobs{1}.spatial{1}.coreg{1}.write.source{r} = fullfile(path, ['sbr' name ext]); %#ok<AGROW>
end
[path, name, ext] = fileparts(roi_fname{1});
[hdr, img] = read_nifti(fullfile(path, ['sbr' name ext]));
hdr.datatype = 2; % uchar
hdr.bitpix = 8;
write_nifti(hdr, img < 5, fullfile(path, ['dilate_' name ext]));
jobs{1}.spatial{1}.coreg{1}.write.source{nrois + 1} = fullfile(path, ['dilate_' name ext]);
spm_jobman('run_nogui', jobs);

% segment anatomical
clear jobs;
[path, name, ext] = fileparts(mri_fname);
jobs{1}.spatial{1}.preproc.data{1} = fullfile(path, ['r' name ext]);
jobs{1}.spatial{1}.preproc.output.GM = [0 0 1];
jobs{1}.spatial{1}.preproc.output.WM = [0 0 1];
jobs{1}.spatial{1}.preproc.output.CSF = [0 0 1];
jobs{1}.spatial{1}.preproc.output.biascor = 0;
jobs{1}.spatial{1}.preproc.output.cleanup = 1;
[path, name, ext] = fileparts(roi_fname{1}); % first ROI used as exclusion mask
jobs{1}.spatial{1}.preproc.opts.msk{1} = fullfile(path, ['rdilate_' name ext]);

% write normalized anatomical and mask images
[path, name, ext] = fileparts(mri_fname);
matfname = fullfile(path, ['r' name '_seg_sn.mat']);
jobs{2}.spatial{1}.normalise{1}.write.subj.matname{1} = matfname;
jobs{2}.spatial{1}.normalise{1}.write.subj.resample{1} = fullfile(path, ['r' name ext]);
for r = 1:nrois
  [path, name, ext] = fileparts(roi_fname{r});
  jobs{2}.spatial{1}.normalise{1}.write.subj.resample{r + 1} = fullfile(path, ['rsbr' name ext]); %#ok<AGROW>
end
jobs{2}.spatial{1}.normalise{1}.write.roptions.vox = [2 2 2];
jobs{2}.spatial{1}.normalise{1}.write.roptions.bb = [-90 -126 -72; 90 90 108];

spm_jobman('run_nogui', jobs);

% divide the mask images by 255
for r = 1:nrois
  [path, name, ext] = fileparts(roi_fname{r});
  [hdr, img] = read_nifti(fullfile(path, ['wrsbr' name ext]));
  hdr.datatype = 16; % float
  hdr.bitpix = 32;
  img = img ./ 255;
  write_nifti(hdr, img, fullfile(path, ['final_' name ext]));
end
